# Generated by Django 2.0.5 on 2018-06-04 16:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('genomix_workflows', '0004_auto_20180601_1216'),
    ]

    operations = [
        migrations.AddField(
            model_name='reference',
            name='workflow_category',
            field=models.CharField(default='order', max_length=100),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='taskinstance',
            name='status',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Pending'), (2, 'Complete'), (3, 'Running'), (4, 'Hold'), (5, 'Skip'), (6, 'Failed')], default=1),
        ),
        migrations.AlterField(
            model_name='workflowinstance',
            name='state',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Active'), (2, 'Complete'), (3, 'Hold'), (4, 'Cancelled'), (5, 'Failed')], default=1),
        ),
    ]
